﻿using System.Collections.Generic;

using VSTSConnector;
using DeviceBridge;

namespace BslWatcher
{
    public class Controller
    {
        private string m_sVSTSURL = "https://my_vsts_web_site.com";
        private string m_sVSTSProject = "my_project_name";

        private string m_sStatus = "started";
        private string m_sCurrentCommand;
        private string m_sCurrentCOMPort;

        private List<BuildInfoItem> m_oBuildInfos;

        BuildInfoMonitor m_oVSTS = null;
        SerialDevice m_oDevice = null;

        private string m_sLastBuildStatus= "Success";

        public Controller()
        {
            m_oDevice = new SerialDevice();

            m_oVSTS = new BuildInfoMonitor(m_sVSTSURL, m_sVSTSProject);
            m_oVSTS.AddBuildToMonitor("build_definition_1");
            m_oVSTS.AddBuildToMonitor("build_definition_2");
            m_oVSTS.AddBuildToMonitor("build_definition_3");
            m_oVSTS.AddBuildToMonitor("build_definition_4");
            m_oVSTS.AddBuildToMonitor("build_definition_5");

            m_sCurrentCommand = GetCommands()[0];
            m_sCurrentCOMPort = GetCOMPorts()[3];

            m_oBuildInfos = new List<BuildInfoItem>();
        }

        public List<string> GetCommands()
        {
            List<string> oList = new List<string>();
            oList.AddRange(DeviceCommands.GetAllCommands());
            return oList;
        }

        public List<string> GetCOMPorts()
        {
            List<string> oList = new List<string>();
            oList.AddRange(SerialPorts.GetAllPorts());
            return oList;
        }

        public string CurrentCOMPort
        {
            get { return m_sCurrentCOMPort; }
            set { m_sCurrentCOMPort = value; }
        }

        public string CurrentCommand
        {
            get { return m_sCurrentCommand; }
            set { m_sCurrentCommand = value; }
        }

        public string Status
        {
            get { return m_sStatus; }
        }

        public string LastBuildStatus
        {
            get { return m_sLastBuildStatus; }
        }

        public List<BuildResult> BuildInfos
        {
            get {
                List<BuildResult> oList = new List<BuildResult>();
                foreach (var item in m_oBuildInfos)
                {
                    oList.Add(new BuildResult(item.Name, item.BuildNumber,
                        item.FinishTime, item.Result));
                }

                return oList;
            }
        }

        public void SendManualCommand(string sCommand)
        {
            m_sCurrentCommand = sCommand;
            m_oDevice.ComPort = SerialPorts.GetPortNbFromString(m_sCurrentCOMPort);
            m_oDevice.SendCommand(DeviceCommands.GetCommandFromString(m_sCurrentCommand));
        }

        public void RefreshData()
        {
            var oCurrentInfos = m_oVSTS.GetBuilds();
            TreatBuildInfos(oCurrentInfos);
            m_oBuildInfos = oCurrentInfos;
        }

        private void TreatBuildInfos(List<BuildInfoItem> oList)
        {
            bool bSuccess = m_oVSTS.IsBuildInError();
            m_oDevice.ComPort = SerialPorts.GetPortNbFromString(m_sCurrentCOMPort);
            uint nCmd = DeviceCommands.CMD_SUCCESS;
            m_sLastBuildStatus = "Success";
            if (bSuccess)
            {
                nCmd = DeviceCommands.CMD_ERROR;
                m_sLastBuildStatus = "Error";
            }
            m_oDevice.SendCommand(nCmd);
        }
    }
}
