﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Threading;
using System.ComponentModel;

namespace BslWatcher
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private BackgroundWorker m_backgroundThread = new BackgroundWorker();

        private DispatcherTimer m_oTimer = new DispatcherTimer();

        private Controller m_oController = new Controller();
        private bool m_bPageLoaded = false;
        public MainWindow()
        {
            InitializeComponent();
            InitializeWorkerThread();
            InitListView();
            InitialState();
            CreateTimer();
            m_bPageLoaded = true;
        }

        private void CreateTimer()
        {
            m_oTimer.Interval = TimeSpan.FromSeconds(2);
            m_oTimer.Tick += OnTimerTick;
            m_oTimer.Start();
        }

        private void OnTimerTick(object sender, EventArgs e)
        {
            m_oTimer.Stop();
            if (!m_backgroundThread.IsBusy)
            {
                UpdateStatusBar("Refreshing list...");
                m_backgroundThread.RunWorkerAsync();
            }
            m_oTimer.Interval = TimeSpan.FromSeconds(30);
            m_oTimer.Start();
        }

        private void RefreshUI()
        {
            m_listView.Items.Clear();
            List<BuildResult> oBuildResults = m_oController.BuildInfos;
            foreach (var oInfo in oBuildResults)
            {
                m_listView.Items.Add(oInfo);
            }
            m_txtLastBuild.Text= "Last Build: "+ m_oController.LastBuildStatus;

            DateTime dtNow = DateTime.Now;
            string sMessage = string.Format(
                            "List refreshed at {0}",
                            dtNow.ToString("T")
                        );
            UpdateStatusBar(sMessage);
        }

        private void UpdateStatusBar(string sMessage)
        {
            m_statusBar.Text = sMessage;
        }

        private void InitialState()
        {
            UpdateStatusBar("");
            InitComPortsListBox();
            InitCommandsListBox();
        }

        private void InitComPortsListBox()
        {
            var comPorts = m_oController.GetCOMPorts();
            foreach (var portName in comPorts)
            {
                m_ComPorts.Items.Add(portName);
            }
            m_ComPorts.SelectedItem = m_oController.CurrentCOMPort;
        }

        private void InitCommandsListBox()
        {
            var commands = m_oController.GetCommands();
            foreach (var item in commands)
            {
                m_Commands.Items.Add(item);
            }
            m_Commands.SelectedItem = m_oController.CurrentCommand;
        }

        private void InitListView()
        {
            var gridView = new GridView();
            m_listView.View = gridView;

            gridView.Columns.Add(new GridViewColumn
            {
                Header = "Result",
                DisplayMemberBinding = new Binding("Result")
            });
            gridView.Columns.Add(new GridViewColumn
            {
                Header = "Finish Time",
                DisplayMemberBinding = new Binding("CompletionDate")
            });
            gridView.Columns.Add(new GridViewColumn
            {
                Header = "Build Name",
                DisplayMemberBinding = new Binding("Name")
            });
            gridView.Columns.Add(new GridViewColumn
            {
                Header = "Build Number",
                DisplayMemberBinding = new Binding("BuildNumber")
            });
        }

        private void OnSelectionChanged_ComPorts(object sender, SelectionChangedEventArgs e)
        {
            if (m_bPageLoaded)
            {
                string sSelection = m_ComPorts.SelectedItem.ToString();
                m_oController.CurrentCOMPort = sSelection;

                string sMessage = string.Format("Change COM Port to '{0}'",
                                                sSelection);
                UpdateStatusBar(sMessage);
            }
        }

        private void OnSelectionChanged_Commands(object sender, SelectionChangedEventArgs e)
        {
            if (m_bPageLoaded)
            {
                string sSelection = m_Commands.SelectedItem.ToString();
                m_oController.SendManualCommand(sSelection);
                string sMessage = string.Format("Send Manual Command: '{0}'",
                                                sSelection);
                UpdateStatusBar(sMessage);
            }
        }

        private void OnBackgroundThreadDoWork(object sender, DoWorkEventArgs e)
        {
            m_oController.RefreshData();
        }

        private void OnBackgroundThreadWorkCompleted(
            object sender,
            RunWorkerCompletedEventArgs e)
        {
            RefreshUI();
        }

        private void InitializeWorkerThread()
        {
            this.m_backgroundThread.DoWork += new System.ComponentModel.DoWorkEventHandler(this.OnBackgroundThreadDoWork);
            this.m_backgroundThread.RunWorkerCompleted += new System.ComponentModel.RunWorkerCompletedEventHandler(this.OnBackgroundThreadWorkCompleted);
        }
    }
}
