﻿using System;
using System.Collections.Generic;
using Microsoft.VisualStudio.Services.WebApi;
using Microsoft.VisualStudio.Services.Common;
using Microsoft.VisualStudio.Services.Client;
using Microsoft.TeamFoundation.Build.WebApi;

namespace VSTSConnector
{
    public class BuildInfoRetriever
    {
        private readonly string m_sURL;
        private readonly string m_sProject;
        private HashSet<string> m_oList= new HashSet<string>();
        public BuildInfoRetriever(string sURL, string sProject)
        {
            m_sURL = sURL;
            m_sProject = sProject;
        }

        public void AddBuildToMonitor(string sName)
        {
            m_oList.Add(sName);
        }

        public List<BuildInfoItem> GetBuildInfo()
        {
            List<BuildInfoItem> oList = new List<BuildInfoItem>();

            var u = new Uri(m_sURL);

            VssCredentials credentials = new VssAadCredential();
            VssConnection connection = new VssConnection(u, credentials);

            BuildHttpClient buildServer = connection.GetClient<BuildHttpClient>();

            //get all builds information in your team projects
            var builds = buildServer.GetBuildsAsync(project: m_sProject).Result;
            foreach (var build in builds)
            {
                //Console.WriteLine(build.Definition.Name + "--" + build.BuildNumber + "--" + build.FinishTime + "--" + build.Result);
                if (IsBuildMonitored(build.Definition.Name))
                {
                    //finish time is not defined for running builds
                    if (build.FinishTime != null)
                    {
                        DateTime oFinishTime = (DateTime)build.FinishTime;
                        BuildInfoItem oItem = new BuildInfoItem(build.Definition.Name,
                                            build.BuildNumber,
                                            oFinishTime,
                                            build.Result.ToString());
                        oList.Add(oItem);
                    }
                }
            }

            return oList;
        }

        private bool IsBuildMonitored(string sName)
        {
            if (0 == m_oList.Count)
            {
                return true;
            }
            return m_oList.Contains(sName);
        }
    }
}
