﻿using System;
using System.Collections.Generic;
using Microsoft.VisualStudio.TestTools.UnitTesting;

using VSTSConnector;

namespace VSTSConnectorTest
{
    [TestClass]
    public class BuildListComparerTest
    {
        [TestMethod]
        public void TestNullParameters()
        {
            BuildListComparer oTst = new BuildListComparer();
            Assert.IsFalse(oTst.CompareList(null, null));
        }

        [TestMethod]
        public void TestFirstParameterNull()
        {
            List<BuildInfoItem> oList = new List<BuildInfoItem>();

            BuildListComparer oTst = new BuildListComparer();
            Assert.IsFalse(oTst.CompareList(null, oList));
        }

        [TestMethod]
        public void TestSecondParameterNull()
        {
            List<BuildInfoItem> oList = new List<BuildInfoItem>();

            BuildListComparer oTst = new BuildListComparer();
            Assert.IsFalse(oTst.CompareList(oList, null));
        }

        [TestMethod]
        public void TestSameList()
        {
            List<BuildInfoItem> oList= new List<BuildInfoItem>{ new BuildInfoItem("name", "build", DateTime.Now, "Succeeded") };

            BuildListComparer oTst = new BuildListComparer();
            Assert.IsFalse(oTst.CompareList(oList, oList));
        }

        [TestMethod]
        public void TestNewItemSuccess()
        {
            List<BuildInfoItem> oOldList = new List<BuildInfoItem> {
                new BuildInfoItem("name1", "build1", DateTime.Now, "Succeeded") };

            List<BuildInfoItem> oNewList = new List<BuildInfoItem> {
                new BuildInfoItem("name1", "build1", DateTime.Now, "Succeeded"),
                new BuildInfoItem("name2", "build2", DateTime.Now.AddSeconds(1), "Succeeded"),
            };

            BuildListComparer oTst = new BuildListComparer();
            Assert.IsFalse(oTst.CompareList(oOldList, oNewList));
        }

        [TestMethod]
        public void TestNewItemSucceeded2()
        {
            List<BuildInfoItem> oOldList = new List<BuildInfoItem> {
                new BuildInfoItem("name1", "build1", DateTime.Now, "Failed") };

            List<BuildInfoItem> oNewList = new List<BuildInfoItem> {
                new BuildInfoItem("name1", "build1", DateTime.Now, "Failed"),
                new BuildInfoItem("name2", "build2", DateTime.Now.AddSeconds(1), "Succeeded"),
            };

            BuildListComparer oTst = new BuildListComparer();
            Assert.IsFalse(oTst.CompareList(oOldList, oNewList));
        }

        [TestMethod]
        public void TestNewItemFailed()
        {
            List<BuildInfoItem> oOldList = new List<BuildInfoItem> {
                new BuildInfoItem("name1", "build1", DateTime.Now, "Succeeded") };

            List<BuildInfoItem> oNewList = new List<BuildInfoItem> {
                new BuildInfoItem("name1", "build1", DateTime.Now, "Succeeded"),
                new BuildInfoItem("name2", "build2", DateTime.Now.AddSeconds(1), "Failed"),
            };

            BuildListComparer oTst = new BuildListComparer();
            Assert.IsTrue(oTst.CompareList(oOldList, oNewList));
        }

        [TestMethod]
        public void TestNewItemFailed2()
        {
            List<BuildInfoItem> oOldList = new List<BuildInfoItem> {
                new BuildInfoItem("name1", "build1", DateTime.Now, "Failed") };

            List<BuildInfoItem> oNewList = new List<BuildInfoItem> {
                new BuildInfoItem("name1", "build1", DateTime.Now, "Failed"),
                new BuildInfoItem("name2", "build2", DateTime.Now.AddSeconds(1), "Failed"),
            };

            BuildListComparer oTst = new BuildListComparer();
            Assert.IsTrue(oTst.CompareList(oOldList, oNewList));
        }
    }
}
